/*******************************************************************
 *
 * OpenLLDP TLV Header
 *
 * Licensed under a dual GPL/Proprietary license.  
 * See LICENSE file for more info.
 * 
 * File: lldp_tlv.h
 * 
 * Authors: Terry Simons (terry.simons@gmail.com)
 *
 *******************************************************************/

#ifndef LLDP_TLV_H
#define LLDP_TLV_H


/* TLV Types from section 9.4.1 of IEEE 802.1AB */
#define END_OF_LLDPDU_TLV       0    /* MANDATORY */
#define CHASSIS_ID_TLV          1    /* MANDATORY */
#define PORT_ID_TLV             2    /* MANDATORY */
#define TIME_TO_LIVE_TLV        3    /* MANDATORY */
#define PORT_DESCRIPTION_TLV    4    /* OPTIONAL  */
#define SYSTEM_NAME_TLV         5    /* OPTIONAL  */
#define SYSTEM_DESCRIPTION_TLV  6    /* OPTIONAL  */
#define SYSTEM_CAPABILITIES_TLV 7    /* OPTIONAL  */
#define MANAGEMENT_ADDRESS_TLV  8    /* OPTIONAL  */
/* 9 - 126 are reserved */
#define ORG_SPECIFIC_TLV        127  /* OPTIONAL */
/* TLV Subtypes from section 9 of IEEE 802.1AB */
#define MED_OUI 0x000012BB
#define ORG802_OUI 0x0000120F
#define ORG801_OUI 0x000080C2

/* Chassis ID TLV Subtypes */
/* 0 is reserved */
#define CHASSIS_ID_CHASSIS_COMPONENT 1
#define CHASSIS_ID_INTERFACE_ALIAS   2
#define CHASSIS_ID_PORT_COMPONENT    3
#define CHASSIS_ID_MAC_ADDRESS       4
#define CHASSIS_ID_NETWORK_ADDRESS   5
#define CHASSIS_ID_INTERFACE_NAME    6
#define CHASSIS_ID_LOCALLY_ASSIGNED  7 
/* 8-255 are reserved */
/* End Chassis ID TLV Subtypes */

/* Port ID TLV Subtypes */
/* 0 is reserved */
#define PORT_ID_INTERFACE_ALIAS  1
#define PORT_ID_PORT_COMPONENT   2
#define PORT_ID_MAC_ADDRESS      3
#define PORT_ID_NETWORK_ADDRESS  4
#define PORT_ID_INTERFACE_NAME   5
#define PORT_ID_AGENT_CIRCUIT_ID 6
#define PORT_ID_LOCALLY_ASSIGNED 7
/* 8-255 are reserved */ 
/* End Port ID TLV Subtypes */

/* System Capabilities TLV Subtypes */
#define SYSTEM_CAPABILITY_OTHER     1
#define SYSTEM_CAPABILITY_REPEATER  2
#define SYSTEM_CAPABILITY_BRIDGE    4
#define SYSTEM_CAPABILITY_WLAN      8
#define SYSTEM_CAPABILITY_ROUTER    16
#define SYSTEM_CAPABILITY_TELEPHONE 32
#define SYSTEM_CAPABILITY_DOCSIS    64
#define SYSTEM_CAPABILITY_STATION   128
/* 8 - 15 reserved */
/* End System Capabilities TLV Subtypes */

/* End TLV Subtypes from section 9 of IEEE 802.1AB */

/* 802.1AB Annex F IEEE 802.1 Organizationally Specific TLVs */
/* 0 reserved */
#define PORT_VLAN_ID_TLV              1  /* Subclause Reference: F.2 */
#define PORT_AND_PROTOCOL_VLAN_ID_TLV 2  /* Subclause Reference: F.3 */
#define VLAN_NAME_TLV                 3  /* Subclause Reference: F.4 */
#define PROTOCOL_IDENTITY_TLV         4  /* Subclause Reference: F.5 */
/* 5 - 255 reserved */
/* End 802.1AB Annex F IEEE 802.1 Organizationally Specific TLVs */

/* IEEE 802.1 Port and Protocol VLAN ID Capability/Status Masks */
/* bit 0 reserved */
#define PORT_AND_PROTOCOL_VLAN_SUPPORTED 2 /* bit 1 */
#define PORT_AND_PROTOCOL_VLAN_ENABLED   4 /* bit 2 */
/* bits 3 - 7 reserved (set to zero) */
/* End IEE 802.1 Port and Protocol VLAN ID Capability/Status Masks */

/* 802.1AB Annex G IEEE 802.3 Organizationally SPecific TLVs */
/* 0 reserved */
#define MAC_PHY_CONFIGURATION_STATUS_TLV 1 /* Subclause Reference: G.2 */
#define POWER_VIA_MDI_TLV                2 /* Subclause Reference: G.3 */
#define LINK_AGGREGATION_TLV             3 /* Subclause Reference: G.4 */
#define MAXIMUM_FRAME_SIZE_TLV           4 /* Subclause Reference: G.5 */
/* 5 - 255 reseved */
/* End 802.1AB Annex G IEEE 802.3 Organizationally Specific TLVs */

/* IEEE 802.3 auto-negotiation Support/Status Masks */
#define AUTO_NEGOTIATION_SUPPORT 1 /* bit 0 */
#define AUTO_NEGOTIATION_STATUS  2 /* bit 1 */
/* bits 2 - 7 reserved */
/* End IEEE 802.3 auto-negotiation Support/Status Masks */

/* IEEE 802.3 Aggregation Capability/Status Masks */
#define AGGREGATION_CAPABILITY 1 /* bit 0 */
#define AGGREGATION_STATUS     2 /* bit 1 */
/* bits 2 - 7 reserved */
/* End IEEE 802.3 Aggregation Capability/Status Masks */

#define bOther   0
#define bAUI     1
#define b10base5 2

struct lldp_flat_tlv {
  uint16 size;
  uint8 *tlv;
};

struct lldp_tlv {
  uint8  type;
  uint16 length;
  uint8  *info_string;
};

struct lldp_tlv_list {
  struct lldp_tlv *tlv;
  struct lldp_tlv_list *next;
};

void add_tlv(struct lldp_tlv *tlv, struct lldp_tlv_list **tlv_list);

struct lldp_flat_tlv *flatten_tlv(struct lldp_tlv *tlv);
struct lldp_tlv *explode_tlv(struct lldp_flat_tlv *flat_tlv);

uint8 validate_generic_tlv(struct lldp_tlv *tlv);
void decode_tlvs(struct lldp_tlv *tlv,struct LldpMib *lldp_mib);

void destroy_tlv(struct lldp_tlv *tlv);
void destroy_flattened_tlv(struct lldp_flat_tlv *tlv);

struct lldp_tlv *create_end_of_lldpdu_tlv();
uint8 validate_end_of_lldpdu_tlv(struct lldp_tlv *tlv);

struct lldp_tlv *create_chassis_id_tlv(struct ChassisId *chassis_id);
uint8 validate_chassis_id_tlv(struct lldp_tlv *tlv);
void decode_tlv_chassisId(struct lldp_tlv *tlv,struct LldpMib *lldp_mib);

struct lldp_tlv *create_port_id_tlv(struct PortId *port_id);
uint8 validate_port_id_tlv(struct lldp_tlv *tlv);
void decode_tlv_portId(struct lldp_tlv *tlv,struct LldpMib *lldp_mib);

struct lldp_tlv *create_ttl_tlv(uint16 txTTL);
uint8 validate_ttl_tlv(struct lldp_tlv *tlv);

struct lldp_tlv *create_system_capabilities_tlv(struct Sys_capabilities *sys_cap);
uint8 validate_system_capabilities_tlv(struct lldp_tlv *tlv);
void decode_tlv_system_capabilities(struct lldp_tlv *tlv,struct LldpMib *lldp_mib);

struct lldp_tlv *create_macPhy_configStatus_tlv(struct MacPhy_configStatus *mac_phy);
uint8 validate_macPhy_configStatus_tlv(struct lldp_tlv *tlv);
void decode_tlv_macPhy_configStatus(struct lldp_tlv *tlv,struct LldpMib *lldp_mib);

struct lldp_tlv *create_medCapabilities_tlv(struct MedCapabilities *med_cap);
uint8 validate_medCapabilities_tlv(struct lldp_tlv *tlv);
void decode_tlv_medCapabilities(struct lldp_tlv *tlv,struct LldpMib *lldp_mib);

struct lldp_tlv *create_networkPolicy_tlv(struct NetworkPolicyTlv *network_policy);
uint8 validate_networkPolicy_tlv(struct lldp_tlv *tlv);
void decode_tlv_networkPolicy(struct lldp_tlv *tlv,struct LldpMib *lldp_mib);

struct lldp_tlv *create_extendedPower_tlv(struct ExtendedPower *ext_power);
uint8 validate_extendedPower_tlv(struct lldp_tlv *tlv);
void decode_tlv_extendedPower(struct lldp_tlv *tlv,struct LldpMib *lldp_mib);

void decode_tlv_portVlanId(struct lldp_tlv *tlv,struct LldpMib *lldp_mib);
void decode_tlv_portAndProtoVlanId(struct lldp_tlv *tlv,struct LldpMib *lldp_mib);
void decode_tlv_vlanName(struct lldp_tlv *tlv,struct LldpMib *lldp_mib);
void decode_tlv_protoIdentity(struct lldp_tlv *tlv,struct LldpMib *lldp_mib);
// Should probably allow this create function to specify the OUI in question?
//struct lldp_tlv *create_organizationally_specific_tlv(struct lldp_port *lldp_port);
uint8 validate_organizationally_specific_tlv(struct lldp_tlv *tlv);

extern uint8 (*validate_tlv[])(struct lldp_tlv *tlv);

#define LLDP_BEGIN_RESERVED_TLV 9
#define LLDP_END_RESERVED_TLV 126

#define XVALIDTLV     0
#define XEINVALIDTLV -1
#define XUNRECOGTLV   1
#define TRUE 1
#define FALSE 0

void decode_oui_tlv(struct lldp_tlv *tlv);
char *capability_name(uint16 capability);
char *tlv_typetoname(uint8 tlv_type);
uint8 tlvcpy(struct lldp_tlv *dst, struct lldp_tlv *src);
struct lldp_tlv *tlvpop(uint8 *buffer);
uint8 tlvpush(uint8 *buffer, struct lldp_tlv *tlv);

uint8 lldp_cache_tlv(struct lldp_tlv *tlv);

uint8 tlvInitializeLLDP();
void tlvCleanupLLDP();
struct lldp_tlv *initialize_tlv();
void destroy_tlv_list(struct lldp_tlv_list **tlv_list);

//struct lldp_msap_cache *lldp_msap_cache(struct lldp_port *lldp_port, uint8 *msap, uint64 msap_length);
//uint8 update_msap_cache(struct lldp_port *lldp_port, struct lldp_msap_cache *msap_cache);
//uint8 create_msap_cache(struct lldp_port *lldp_port, uint8 *msap, uint64 msap_length, struct lldp_tlv_list *tlv_list);
//void destroy_msap_cache(struct lldp_msap_cache *msap_cache);

uint8 initializeTLVFunctionValidators();


#endif /* LLDP_TLV_H */
