/*******************************************************************
 *
 * OpenLLDP TLV
 *
 * Licensed under a dual GPL/Proprietary license.  
 * See LICENSE file for more info.
 * 
 * File: lldp_tlv.c
 * 
 * Authors: Terry Simons (terry.simons@gmail.com)
 *
 *******************************************************************/

#include <arpa/inet.h>
#include <stdlib.h>
#include <string.h>
#ifdef DEBUG_ENABLED
#include <stdio.h>
#endif
#include "ifx_common_defs.h"
#include "ifx_lldp_config.h"
#include "ifx_lldp_cm.h"
#include "ifx_lldp_tx_sm.h"
#include "ifx_lldp_rx_sm.h"
#include "ifx_lldp_mib_if.h"
#include "ifx_lldp_rx_module.h"
#include "ifx_lldp_tx_module.h"
#include "ifx_lldp_port.h"
#include "ifx_lldp_tlv.h"
#include "ifx_lldp_neighbor.h"

/* There are a max of 128 TLV validators (types 0 through 127), so we'll stick
 * them in a static array indexed by their tlv type */
uint8 (*validate_tlv[128])(struct lldp_tlv *tlv) = {
  validate_end_of_lldpdu_tlv,        /* 0 End of LLDPU TLV        */
  validate_chassis_id_tlv,          /* 1 Chassis ID TLV          */
  validate_port_id_tlv,             /* 2 Port ID TLV             */
  validate_ttl_tlv,                 /* 3 Time To Live TLV        */
  NULL,                             /* 4 Port Description TLV    */
  NULL,                             /* 5 System Name TLV         */
  NULL,                             /* 6 System Description TLV  */
  validate_system_capabilities_tlv, /* 7 System Capabilities TLV */
  NULL,                             /* 8 Management Address TLV  */
  /* 9 - 126 are reserved and set to NULL in lldp_tlv_validator_init()                        */
  /* 127 is populated for validate_organizationally_specific_tlv in lldp_tlv_validator_init() */
};

void decode_tlvs(struct lldp_tlv *tlv,struct LldpMib *lldp_mib)
{
  switch(tlv->type)
    {
    case CHASSIS_ID_TLV:
      {
        decode_tlv_chassisId(tlv,lldp_mib);	
      }break;
    case PORT_ID_TLV:
      {
        decode_tlv_portId(tlv,lldp_mib);
      }break;
    case TIME_TO_LIVE_TLV:
      {
//	    debug_printf(DEBUG_TLV, "Time To Live: %d seconds\n", htons(*ttl));
      }break;
    case SYSTEM_CAPABILITIES_TLV:
      {
        decode_tlv_system_capabilities(tlv,lldp_mib);
      }break;
    case ORG_SPECIFIC_TLV:
      {
        uint32_t medHdr = htonl(*(uint32_t *)&tlv->info_string[0]);
		uint32_t oui=0;
		uint32_t subtype=0;
		//fetching the Organizational Unique Identifier from MED header
	    oui = medHdr & 0xFFFFFF00;
		oui=oui>>8;
		//extracting the subtype
        subtype = medHdr & 0x000000FF;
		if (oui==MED_OUI)
		{
          if(subtype == 1)
          {
		    // LLDP-MED Capabilities TLV
			decode_tlv_medCapabilities(tlv,lldp_mib);
		  } else if(subtype == 2){
		    // Network Policy TLV
		    decode_tlv_networkPolicy(tlv,lldp_mib);
		  } else if (subtype == 4){
		    // Extended Power-via-MDI TLV
		    decode_tlv_extendedPower(tlv,lldp_mib); 
		  } else {
		    PRINT("\n This MED TLV is not supported\n");
		  }		  
		} else if (oui==ORG802_OUI) {
		  if(subtype == 1)
		  {
		    // It's MAC/PHY Configuration/Status TLV
            decode_tlv_macPhy_configStatus(tlv,lldp_mib);
		  } else {
		    PRINT("\n This Organizational TLV is not supported\n");
		  }
    } else if (oui==ORG801_OUI){
      if (subtype == 1)
      {
        //It's Port VLAN ID
        decode_tlv_portVlanId(tlv,lldp_mib);
      } else if(subtype == 2) {
        //It's Port and protocol vlan
        decode_tlv_portAndProtoVlanId(tlv,lldp_mib);
      } else if(subtype == 3) {
        //It's vlan name
        decode_tlv_vlanName(tlv,lldp_mib);
      } else if(subtype == 4) {
        // It's protocol identity
        decode_tlv_protoIdentity(tlv,lldp_mib);
      } else {
        PRINT("\n This Organizational TLV is not supported\n");
      }

		} else {
		  PRINT("\n This Organizational TLV is not supported\n");
		}
		
      }break;

     default:
//      debug_printf(DEBUG_NORMAL, "Got unrecognized type '%d'\n", tlv->type);
     break;
    }
}

void decode_tlv_chassisId (struct lldp_tlv *tlv,struct LldpMib *lldp_mib) 
{
  uint8 *chassisId;
  lldp_mib->chassis_id.subType=*(uint8 *)&tlv->info_string[0];
  chassisId = (uint8 *)(&tlv->info_string[0]+sizeof(uint8));
  memset (lldp_mib->chassis_id.chassisId,'\0',255);
  memcpy( lldp_mib->chassis_id.chassisId,chassisId,tlv->length-1);

}
void decode_tlv_portId(struct lldp_tlv *tlv,struct LldpMib *lldp_mib)
{
  uint8 *portId;
  lldp_mib->port_id.subType=*(uint8 *)&tlv->info_string[0];
  portId = (uint8 *)(&tlv->info_string[0]+sizeof(uint8));
  memset (lldp_mib->port_id.portId,'\0',255);
  memcpy( lldp_mib->port_id.portId,portId,tlv->length-1);
            PRINT("######port id is %d\n",lldp_mib->port_id.portId[0]);
}
void decode_tlv_medCapabilities(struct lldp_tlv *tlv,struct LldpMib *lldp_mib)
{
  /*starting address of this MED TLV info string is tlv 
  info_string starting addr + MED header length */
  lldp_mib->med_cap.capSupported = 
    htons(*(uint16_t *)(&tlv->info_string[0] + sizeof(uint32_t)));
  lldp_mib->med_cap.deviceType = 
    *(uint8 *)(&tlv->info_string[0]+sizeof(uint32_t)+sizeof(uint16_t));
}
void decode_tlv_macPhy_configStatus(struct lldp_tlv *tlv,struct LldpMib *lldp_mib)
{
  /*starting address of this Organizational specific TLV info string is 
  tlv info_string starting addr + OUI header length */
  uint8 autoNegFlags;
  autoNegFlags=*(uint8 *)(&tlv->info_string[0]+sizeof(uint32_t));
  /*first bit represents Auto-negotiation support*/
  lldp_mib->mac_phy.autoNegSupported = autoNegFlags & 0X01;
  /*second bit represents Auto-negotiation status*/
  lldp_mib->mac_phy.autoNegEnabled = autoNegFlags & 0X02;
  lldp_mib->mac_phy.autoNegEnabled = lldp_mib->mac_phy.autoNegEnabled >> 1;
  lldp_mib->mac_phy.autoNegAdvCap = 
    htons(*(uint16_t *)(&tlv->info_string[0]+sizeof(uint32_t)+sizeof(autoNegFlags)));
  lldp_mib->mac_phy.operMauType =
    htons(*(uint16_t *)(&tlv->info_string[0]+sizeof(uint32_t)+sizeof(autoNegFlags)+sizeof(uint16_t)));  
}
void decode_tlv_networkPolicy(struct lldp_tlv *tlv,struct LldpMib *lldp_mib)
{
  uint8 i=lldp_mib->network_policy.noOfNetworkPolicyTlvs;
  /*starting address of this MED TLV info string is tlv
  info_string starting addr + MED header length */
  if (i<MAXNETWORKTLVS-1)
  {
  uint32_t infoStr = htonl(*(uint32_t *)(&tlv->info_string[0]+sizeof(uint32_t)));
  /*picking the most significant first octet*/
  lldp_mib->network_policy.network_policy_array[i].appType = (uint8)(infoStr >> 24);
  infoStr = infoStr << 8;
  /*next three bits constitutes to next three flags of the TLV*/
  lldp_mib->network_policy.network_policy_array[i].unknownFlag = (uint8)(infoStr >>31);
  infoStr = infoStr << 1;
  lldp_mib->network_policy.network_policy_array[i].taggedFlag = (uint8)(infoStr >>31);
  /*shifting once for tagged flag and once for reserved flag*/
  infoStr = infoStr << 2;
  lldp_mib->network_policy.network_policy_array[i].vLanId = (uint16_t)(infoStr >> 20);
  infoStr = infoStr << 12;
  lldp_mib->network_policy.network_policy_array[i].priority = (uint8)(infoStr >> 29);
  infoStr = infoStr << 3;
  lldp_mib->network_policy.network_policy_array[i].dscpValue = (uint8)(infoStr >> 26);
  lldp_mib->network_policy.noOfNetworkPolicyTlvs++;
  }
}
void decode_tlv_extendedPower(struct lldp_tlv *tlv,struct LldpMib *lldp_mib)
{
  /*starting address of this MED TLV info string is tlv
  info_string starting addr + MED header length */
  uint8 infoStr=0;
  infoStr=*(uint8 *)(&tlv->info_string[0]+sizeof(uint32_t));
  lldp_mib->ext_power.powerPriority = infoStr & 0x0f;
  infoStr=infoStr>>4;
  lldp_mib->ext_power.powerSource = infoStr & 0x03;
  infoStr=infoStr>>2;
  lldp_mib->ext_power.powerType = infoStr & 0x03;

  lldp_mib->ext_power.powerValue =
    htons(*(uint16_t *)(&tlv->info_string[0]+sizeof(uint32_t)+sizeof(infoStr)));
}

void decode_tlv_portVlanId(struct lldp_tlv *tlv,struct LldpMib *lldp_mib)
{
  lldp_mib->port_vlan_id.portVlanId = 
    htons(*(uint16_t *)(&tlv->info_string[0] + sizeof(uint32_t)));
}
void decode_tlv_portAndProtoVlanId(struct lldp_tlv *tlv,struct LldpMib *lldp_mib)
{
  uint8 flags;
  flags=*(uint8 *)(&tlv->info_string[0]+sizeof(uint32_t));
  /*first bit represents Auto-negotiation support*/
  lldp_mib->port_proto_vlan.protoVlanSupported = flags & 0X02;
  /*second bit represents Auto-negotiation status*/
  lldp_mib->port_proto_vlan.protoVlanEnabled = flags & 0X04;
  lldp_mib->port_proto_vlan.protoVlanEnabled = lldp_mib->port_proto_vlan.protoVlanEnabled >> 2;
  lldp_mib->port_proto_vlan.protoVlanId = 
    htons(*(uint16_t *)(&tlv->info_string[0]+sizeof(uint32_t)+sizeof(flags)));
}

void decode_tlv_vlanName(struct lldp_tlv *tlv,struct LldpMib *lldp_mib)
{
  uint8 *vlanName;
  lldp_mib->vlan_name.vlanId = 
    htons(*(uint16_t *)(&tlv->info_string[0] + sizeof(uint32_t)));
  lldp_mib->vlan_name.vlanNameLength=*(uint8 *)(&tlv->info_string[0]+sizeof(uint32_t)+sizeof(uint16_t));
  vlanName=(uint8 *)(&tlv->info_string[0]+sizeof(uint32_t)+sizeof(uint16_t)+sizeof(uint8));
  memcpy( lldp_mib->vlan_name.vlanName,vlanName,lldp_mib->vlan_name.vlanNameLength);
 
}
void decode_tlv_protoIdentity(struct lldp_tlv *tlv,struct LldpMib *lldp_mib)
{
  uint8 *protoId;
  lldp_mib->proto_id.protoIdLength=*(uint8 *)(&tlv->info_string[0]+sizeof(uint32_t));
  protoId=(uint8 *)(&tlv->info_string[0]+sizeof(uint32_t)+sizeof(uint8));
  memcpy( lldp_mib->proto_id.protoId,protoId,lldp_mib->proto_id.protoIdLength);
}
void decode_tlv_system_capabilities(struct lldp_tlv *tlv,struct LldpMib *lldp_mib)
{
  int capability = 0;
  
  uint16_t system_capabilities = htons(*(uint16_t *)&tlv->info_string[0]);
  uint16_t enabled_capabilities = htons(*(uint16_t *)&tlv->info_string[2]);
  lldp_mib->sys_cap.sys_capabilities = system_capabilities;
  lldp_mib->sys_cap.enabled_capabilities = enabled_capabilities;
  //debug_printf(DEBUG_TLV, "System Capabilities: %d, Enabled Capabilities: %d\n", system_capabilities, enabled_capabilities);

  //debug_printf(DEBUG_TLV, "Capabilities:\n");

  for(capability = 1; capability <= 16; capability *= 2)
    {
      if((system_capabilities & capability))
	{
	  if(enabled_capabilities & capability)
	    {
	      //debug_printf(DEBUG_TLV, "%s (enabled)\n", capability_name(capability));
	    }
	  else
	    {
	      //debug_printf(DEBUG_TLV, "%s (disabled)\n", capability_name(capability));
	    }
	} 
    }
}
#if 1
char *capability_name(uint16_t capability)
{
  switch(capability)
    {
    case SYSTEM_CAPABILITY_OTHER:
      return "Other";
    case SYSTEM_CAPABILITY_REPEATER:
      return "Repeater/Hub";
    case SYSTEM_CAPABILITY_BRIDGE:
      return "Bridge/Switch";
    case SYSTEM_CAPABILITY_WLAN:
      return "Wireless LAN";
    case SYSTEM_CAPABILITY_ROUTER:
      return "Router";
    case SYSTEM_CAPABILITY_TELEPHONE:
      return "Telephone";
    case SYSTEM_CAPABILITY_DOCSIS:
      return "DOCSIS/Cable Modem";
    case SYSTEM_CAPABILITY_STATION:
      return "Station";
    default:
      return "Unknown";
    };
}


char *tlv_typetoname(uint8 tlv_type)
{
  switch(tlv_type)
    {
    case CHASSIS_ID_TLV:
      return "Chassis ID";
      break;
    case PORT_ID_TLV:
      return "Port ID";
      break;
    case TIME_TO_LIVE_TLV:
      return "Time To Live";
      break;
    case PORT_DESCRIPTION_TLV:
      return "Port Description";
      break;
    case SYSTEM_NAME_TLV:
      return "System Name";
      break;
    case SYSTEM_DESCRIPTION_TLV:
      return "System Description";
      break;
    case SYSTEM_CAPABILITIES_TLV:
      return "System Capabiltiies";
      break;
    case MANAGEMENT_ADDRESS_TLV:
      return "Management Address";     
      break;
    case ORG_SPECIFIC_TLV:
      return "Organizationally Specific";
      break;
    case END_OF_LLDPDU_TLV:
      return "End Of LLDPDU";
      break;
    default:
      return "Unknown"; 
    };
}
#endif
uint8 tlvInitializeLLDP(struct lldp_port *lldp_port)
{
  /* Initialize the MSAP cache... */
  //lldp_port->rx.msap = NULL;

  /*if(lldp_port->rx.msap == NULL)
    {
      //debug_printf(DEBUG_NORMAL, "[ERROR] Unable to malloc buffer in %s() at line: %d!\n", __FUNCTION__, __LINE__);
    }
  */

  return 0;
}

void tlvCleanupLLDP()
{
}


uint8 initializeTLVFunctionValidators()
{
  int index = 0;

  /* Set all of the reserved TLVs to NULL validator functions */
  /* so they're forced to go through the generic validator    */ 
  for(index = LLDP_BEGIN_RESERVED_TLV; index <= LLDP_END_RESERVED_TLV; index++)
    {
      //debug_printf(DEBUG_EXCESSIVE, "Setting TLV Validator '%d' to NULL - it's reserved\n", index);

      validate_tlv[index] = NULL;
    }

  //debug_printf(DEBUG_EXCESSIVE, "Setting TLV Validator '%d' - it's the organizational specific TLV validator...\n", ORG_SPECIFIC_TLV);

  validate_tlv[ORG_SPECIFIC_TLV] = validate_organizationally_specific_tlv;

  return 0;
}

struct lldp_tlv *initialize_tlv() {
  struct lldp_tlv *tlv = malloc(sizeof(struct lldp_tlv));

  if(tlv) {
    bzero(tlv, sizeof(struct lldp_tlv));
    
    return tlv;  
  }

  return NULL;
}

void destroy_tlv(struct lldp_tlv *tlv) {
  free(tlv->info_string);

  bzero(tlv, sizeof(struct lldp_tlv));
  free(tlv);
  tlv = NULL;
}

void destroy_flattened_tlv(struct lldp_flat_tlv *tlv) {
  free(tlv->tlv);
  tlv->tlv = NULL;
  free(tlv);
}

struct lldp_tlv *create_end_of_lldpdu_tlv(struct lldp_port *lldp_port) {

  struct lldp_tlv* tlv = initialize_tlv();

  tlv->type = END_OF_LLDPDU_TLV; // Constant defined in lldp_tlv.h
  tlv->length = 0;     // The End of LLDPDU TLV is length 0.

  tlv->info_string = NULL;

  return tlv;
}

uint8 validate_end_of_lldpdu_tlv(struct lldp_tlv *tlv)
{
  if(tlv->length != 0)
    { 
      //debug_printf(DEBUG_NORMAL, "[ERROR] TLV type is 'End of LLDPDU' (0), but TLV length is %d when it should be 0!\n", tlv->length);

      return XEINVALIDTLV;
    }
  
  return XVALIDTLV;
}

uint8 validate_length_max_255(struct lldp_tlv *tlv)
{
  //Length will never be below 0 because the variable used is unsigned... 
  if(tlv->length > 255)
    {
      //debug_printf(DEBUG_NORMAL, "[ERROR] TLV has invalid length '%d'.\n\tIt should be between 0 and 255 inclusive!\n", tlv->length);
      
      return XEINVALIDTLV;
    }
  
  return XVALIDTLV;
}

uint8 validate_length_max_256(struct lldp_tlv *tlv)
{
  if(tlv->length < 2 || tlv->length > 256)
    {
      //debug_printf(DEBUG_NORMAL, "[ERROR] TLV has invalid length '%d'.\n\tIt should be between 2 and 256 inclusive!\n", tlv->length);
      
      return XEINVALIDTLV;
    }
  
  return XVALIDTLV;
}

struct lldp_tlv *create_chassis_id_tlv_invalid_length(struct lldp_port *lldp_port) {

  struct lldp_tlv* tlv = initialize_tlv();

  tlv->type = CHASSIS_ID_TLV; // Constant defined in lldp_tlv.h
  tlv->length = 7; //The size of a MAC + the size of the subtype (1 byte)

  tlv->info_string = malloc(tlv->length);  

  // CHASSIS_ID_MAC_ADDRESS is a 1-byte value - 4 in this case. Defined in lldp_tlv.h
  tlv->info_string[0] = CHASSIS_ID_MAC_ADDRESS;

  // We need to start copying at the 2nd byte, so we use [1] here...
  // This reads "memory copy to the destination at the address of tlv->info_string[1] with the source my_mac for 6 bytes" (the size of a MAC address)
  //memcpy(&tlv->info_string[1], &lldp_port->source_mac[0], 6); //COMMENT XXX

  return tlv;
}

struct lldp_tlv *create_chassis_id_tlv(struct ChassisId *chassis_id) {

  struct lldp_tlv* tlv = initialize_tlv();

  tlv->type = CHASSIS_ID_TLV; // Constant defined in lldp_tlv.h
  PRINT("\nIN-> create_chassis_id_tlv , Chassis subType %X\n",chassis_id->subType);
  if ( chassis_id->subType == CHASSISID_SUBTYPE_NETWORK_ADDRESS)
	
	if( chassis_id->chassisId[0] == 0x01 ){
    	tlv->length = 1 + 5; //The size of a IPv4 address + the size of the subtype (1 byte) + IANA Address family (1 byte)
	}else{
		/* Assume IPv6 length */
    	tlv->length = 1 + 17; //The size of a IPv6 address + the size of the subtype (1 byte) + IANA Address family (1 byte)
	}
  else if ( chassis_id->subType == CHASSISID_SUBTYPE_MAC_ADDRESS)
    tlv->length = 1 + 6; //The size of a MAC + the size of the subtype (1 byte)
  else
    tlv->length = 1+ strlen(chassis_id->chassisId); //The size of a MAC + the size of the subtype (1 byte)

  tlv->info_string = malloc(tlv->length);  

  // CHASSIS_ID_MAC_ADDRESS is a 1-byte value - 4 in this case. Defined in lldp_tlv.h
  tlv->info_string[0] = chassis_id->subType;
  PRINT("\nIN-> create_chassis_id_tlv , Chassis ID%d\n",chassis_id->subType);

  // We need to start copying at the 2nd byte, so we use [1] here...
  // This reads "memory copy to the destination at the address of tlv->info_string[1] with the source my_mac for 6 bytes" (the size of a MAC address)
  memcpy(&tlv->info_string[1], chassis_id->chassisId, (tlv->length) - 1);
  PRINT("\n OUT-> create_chassis_id_tlv\n");
  return tlv;
}

uint8 validate_chassis_id_tlv(struct lldp_tlv *tlv)
{
  // Several TLVs have this requirement.
  return validate_length_max_256(tlv);
}

struct lldp_tlv *create_port_id_tlv(struct PortId *port_id) {

  struct lldp_tlv* tlv = initialize_tlv();
  tlv->type = PORT_ID_TLV; // Constant defined in lldp_tlv.h
  if ( port_id->subType == PORTID_SUBTYPE_MAC_ADDRESS)
    tlv->length = 1 + 6; //The length of the interface name + the size of the subtype (1 byte)
  else if ( port_id->subType == PORTID_SUBTYPE_NETWORK_ADDRESS)
    	   tlv->length = 1 + 4; //The length of the interface name + the size of the subtype (1 byte)
       else
    		tlv->length = 1 + strlen(port_id->portId); //The length of the interface name + the size of the subtype (1 byte)

  //debug_printf(DEBUG_NORMAL, "Port Name (%d, %d): %s\n", tlv->length, strlen(lldp_port->if_name), lldp_port->if_name);

  PRINT("\nIN-> create_port_id_tlv subType %X length %X \n", port_id->subType, tlv->length );

  tlv->info_string = malloc(tlv->length);
  bzero(tlv->info_string, tlv->length);

  // PORT_ID_INTERFACE_NAME is a 1-byte value - 5 in this case. Defined in lldp_tlv.h
  tlv->info_string[0] = port_id->subType;


    // We need to start copying at the 2nd byte, so we use [1] here...
    // This reads "memory copy to the destination at the address of tlv->info_string[1] with the source lldp_port->if_name for strlen(lldp_port->if_name) bytes"
  memcpy(&tlv->info_string[1], port_id->portId, (tlv->length) - 1);
  PRINT("\ncreate_port_id_tlv: port id subtype: %x\n",  (uchar8)tlv->info_string[0]);
//  debug_hex_dump(DEBUG_NORMAL, lldp_port->if_name, strlen(lldp_port->if_name));
  //debug_hex_dump(DEBUG_NORMAL, tlv->info_string, tlv->length);

  //debug_printf(DEBUG_NORMAL, "Port Name (%d, %d): %s\n", tlv->length, strlen(lldp_port->if_name), lldp_port->if_name);  


 return tlv;
}

uint8 validate_port_id_tlv(struct lldp_tlv *tlv)
{
  // Several TLVs have this requirement.
  return validate_length_max_256(tlv);
}

struct lldp_tlv *create_ttl_tlv(uint16_t txTTL) {

  struct lldp_tlv* tlv = initialize_tlv();
  uint16_t ttl = htons(txTTL);

  tlv->type = TIME_TO_LIVE_TLV; // Constant defined in lldp_tlv.h
  tlv->length = 2; // Static length defined by IEEE 802.1AB section 9.5.4

  tlv->info_string = malloc(tlv->length);

  memcpy(tlv->info_string, &ttl, tlv->length);

  return tlv;
}

uint8 validate_ttl_tlv(struct lldp_tlv *tlv)
{
  if(tlv->length != 2)
    {
      //debug_printf(DEBUG_NORMAL, "[ERROR] TLV has invalid length '%d'.\n\tLength should be '2'.\n", tlv->length);

      return XEINVALIDTLV;
    }

  return XVALIDTLV;
}

struct lldp_tlv *create_system_capabilities_tlv(struct Sys_capabilities *sys_cap) {
  struct lldp_tlv* tlv = initialize_tlv();

  tlv->type = SYSTEM_CAPABILITIES_TLV; // Constant defined in lldp_tlv.h
  //tlv->type = 120; // Constant defined in lldp_tlv.h

  tlv->length = 4;

  PRINT("\nIN-> create_system_capabilities_tlv , sys_cap->sys_capabilities %d\n",sys_cap->sys_capabilities);
  PRINT("\nIN-> create_system_capabilities_tlv , sys_cap->enabled_capabilities %d\n",sys_cap->enabled_capabilities);
  tlv->info_string = malloc(tlv->length);
  sys_cap->sys_capabilities = htons(sys_cap->sys_capabilities);
  sys_cap->enabled_capabilities = htons(sys_cap->enabled_capabilities);
  memcpy(&tlv->info_string[0], &(sys_cap->sys_capabilities), sizeof(uint16_t));
  memcpy(&tlv->info_string[2], &(sys_cap->enabled_capabilities), sizeof(uint16_t));

  return tlv;
}

uint8 validate_system_capabilities_tlv(struct lldp_tlv *tlv)
{
  if(tlv->length != 4)
    {
      //debug_printf(DEBUG_NORMAL, "[ERROR] TLV has invalid length '%d'.\n\tLength should be '4'.\n", tlv->length);

      return XEINVALIDTLV;
    }

  return XVALIDTLV;
}

struct lldp_tlv *create_macPhy_configStatus_tlv(struct MacPhy_configStatus *mac_phy)
{
  struct lldp_tlv* tlv = initialize_tlv();
  // Organizationally Unique Identifier for the TLV
  uint8 oui[3]={0x00,0x12,0x0F}; 
  //Subtype according to 802.3 spec
  uint32_t subtype =0x01;
  //auto-negotiation support/status
  uint8 autoNeg =0;
  tlv->type = ORG_SPECIFIC_TLV; // Constant defined in lldp_tlv.h
  //OUI + subtype + auto-negotition support + PMD + operational MAU
  tlv->length =9;

  tlv->info_string = malloc(tlv->length);
  uint32_t medHdr=0;
  uint8 i;
  for(i=0;i<3;i++) {
    medHdr=medHdr|oui[i];
    medHdr=medHdr<<8;
  }
  medHdr=medHdr|subtype;
  medHdr=htonl(medHdr);
  memcpy(&tlv->info_string[0], &medHdr, sizeof(uint32_t));

  if(mac_phy->autoNegSupported == TRUE)
    autoNeg=autoNeg|0x01;
  if (mac_phy->autoNegEnabled == TRUE)
    autoNeg=autoNeg|0x02;
  memcpy(&tlv->info_string[4], &autoNeg, sizeof(uint8));
  mac_phy->autoNegAdvCap= htons(mac_phy->autoNegAdvCap);
  mac_phy->operMauType = htons(mac_phy->operMauType);

  memcpy(&tlv->info_string[5], &mac_phy->autoNegAdvCap, sizeof(uint16_t));
  memcpy(&tlv->info_string[7], &mac_phy->operMauType, sizeof(uint16_t));
  return tlv;

}

uint8 validate_macPhy_configStatus_tlv(struct lldp_tlv *tlv)
{
  if(tlv->length != 9)
    {
      //debug_printf(DEBUG_NORMAL, "[ERROR] TLV has invalid length '%d'.\n\tLength should be '9'.\n", tlv->length);

      return XEINVALIDTLV;
    }

  return XVALIDTLV;
}

struct lldp_tlv *create_medCapabilities_tlv(struct MedCapabilities *med_cap) {
  struct lldp_tlv* tlv = initialize_tlv();

  // Organizationally Unique Identifier for the TLV
  uint8 oui[3]={0x00,0x12,0xBB}; 
  //Subtype according to ANSI/TIA-1057 spec
  uint32_t subtype =0x01;

  tlv->type = ORG_SPECIFIC_TLV; // Constant defined in lldp_tlv.h

  tlv->length = 7;

  tlv->info_string = malloc(tlv->length);

  uint32_t medHdr=0;
  uint8 i;
  for(i=0;i<3;i++) {
    medHdr=medHdr|oui[i];
    medHdr=medHdr<<8;
  }
  medHdr=medHdr|subtype;
  medHdr=htonl(medHdr);
  memcpy(&tlv->info_string[0], &medHdr, sizeof(uint32_t));

  med_cap->capSupported = htons(med_cap->capSupported);  	
  memcpy(&tlv->info_string[4], &(med_cap->capSupported), sizeof(uint16_t));
  memcpy(&tlv->info_string[6], &(med_cap->deviceType), sizeof(uint8));

  return tlv;
}

uint8 validate_medCapabilities_tlv(struct lldp_tlv *tlv)
{
  if(tlv->length != 7)
    {
      //debug_printf(DEBUG_NORMAL, "[ERROR] TLV has invalid length '%d'.\n\tLength should be '7'.\n", tlv->length);

      return XEINVALIDTLV;
    }

  return XVALIDTLV;
}

struct lldp_tlv *create_networkPolicy_tlv(struct NetworkPolicyTlv *network_policy) {
  struct lldp_tlv* tlv = initialize_tlv();

  // Organizationally Unique Identifier for the TLV
  uint8 oui[3]={0x00,0x12,0xBB}; 
  //Subtype according to ANSI/TIA-1057 spec
  uint32_t subtype =0x02;
  //The entire network policy info string
  uint32_t infoStr = 0;  
  uint32_t temp=0;
  tlv->type = ORG_SPECIFIC_TLV; // Constant defined in lldp_tlv.h

  tlv->length = 8;

  tlv->info_string = malloc(tlv->length);
  
  uint32_t medHdr=0;
  uint8 i;
  for(i=0;i<3;i++) {
    medHdr=medHdr|oui[i];
    medHdr=medHdr<<8;
  }
  medHdr=medHdr|subtype;
  medHdr=htonl(medHdr);
  memcpy(&tlv->info_string[0], &medHdr, sizeof(uint32_t));
  
  //setting the first byte in network info string
  temp = (temp | network_policy->appType) << 24;
  infoStr=infoStr | temp;
  temp = 0;
  //setting the unknown flag
  temp = temp | network_policy->unknownFlag;
  temp=temp << 23;
  infoStr=infoStr | temp;
  //setting the tagged flag
  temp=0;
  temp=temp | network_policy->taggedFlag;
  temp=temp << 22;
  infoStr=infoStr | temp;
 
  PRINT("\n create_network_policy_tlv unknown %d tagged %d vlanid %d priority %d dcsp\n", network_policy->unknownFlag, network_policy->taggedFlag, network_policy->vLanId, network_policy->priority, network_policy->dscpValue );
  if (network_policy->unknownFlag == FALSE)
  {
    if (network_policy->taggedFlag == TRUE) {
      //setting the VLAN ID
      if (network_policy->vLanId > 4095)
      {
        PRINT("\n VLAN ID not within the range\n");
      } else{
        temp=0;
        temp=network_policy->vLanId;
        temp = temp << 9;
	infoStr=infoStr | temp;	
      }
      if (network_policy->priority > 7)
      {
        PRINT("\n Priority not within the range\n");
      }
      else
      {
        //setting Layer 2 priority
        temp=0;
        temp=network_policy->priority;
        temp = temp << 6;
        infoStr = infoStr | temp; 
      }
    }
    //setting the DSCP value
    if (network_policy->dscpValue > 63)
    {
      PRINT("\n DSCP value not within the range\n");
    }
    else
    {
      temp = 0;
      temp= network_policy->dscpValue;
    //  infoStr=infoStr<<6;
      infoStr=infoStr | temp;
    }
  }
  infoStr=htonl(infoStr);	
  PRINT("network policy info string:%d)))))))))))))))))))))))))))))))))\n", infoStr);
  memcpy(&tlv->info_string[4], &(infoStr), sizeof(uint32_t));

  return tlv;
}

uint8 validate_networkPolicy_tlv(struct lldp_tlv *tlv)
{
  if(tlv->length != 8)
    {
      //debug_printf(DEBUG_NORMAL, "[ERROR] TLV has invalid length '%d'.\n\tLength should be '8'.\n", tlv->length);

      return XEINVALIDTLV;
    }

  return XVALIDTLV;
}

struct lldp_tlv *create_extendedPower_tlv(struct ExtendedPower *ext_power) {
  struct lldp_tlv* tlv = initialize_tlv();

  // Organizationally Unique Identifier for the TLV
  uint8 oui[3]={0x00,0x12,0xBB};
  //Subtype according to ANSI/TIA-1057 spec
  uint32_t subtype =0x04;
  // power info string
  uint8 infoStr=0;
  //temporary variable for individual bit setting
  uint8 temp=0;
  uint16_t powerVal = htons(ext_power->powerValue);
  tlv->type = ORG_SPECIFIC_TLV; // Constant defined in lldp_tlv.h

  tlv->length = 7;

  tlv->info_string = malloc(tlv->length);

  uint32_t medHdr=0;
  uint8 i;
  for(i=0;i<3;i++) {
    medHdr=medHdr|oui[i];
    medHdr=medHdr<<8;
  }
  medHdr=medHdr|subtype;
  medHdr=htonl(medHdr);
  memcpy(&tlv->info_string[0], &medHdr, sizeof(uint32_t));
  //setting Power Type
  if(ext_power->powerType > 3){
    PRINT("\n Unknown Power type\n");
  }
  else{
    infoStr=ext_power->powerType;
  }
  //setting Power Source
  temp=ext_power->powerSource;
  infoStr=infoStr<<2;
  infoStr=infoStr|temp;
  //setting power priority
  temp=0;
  temp=ext_power->powerPriority;
  infoStr=infoStr<<4;
  infoStr=infoStr|temp;
  memcpy(&tlv->info_string[4], &(infoStr), sizeof(uint8));
  memcpy(&tlv->info_string[5], &(powerVal), sizeof(uint16_t));

  return tlv;
}

uint8 validate_extendedPower_tlv(struct lldp_tlv *tlv)
{
  if(tlv->length != 7)
    {
      //debug_printf(DEBUG_NORMAL, "[ERROR] TLV has invalid length '%d'.\n\tLength should be '7'.\n", tlv->length);

      return XEINVALIDTLV;
    }

  return XVALIDTLV;
}

uint8 validate_organizationally_specific_tlv(struct lldp_tlv *tlv)
{
  uint8 ret=XVALIDTLV;
  if(tlv->length > 511)
    {
      //debug_printf(DEBUG_NORMAL, "[ERROR] TLV has invalid length '%d'.\n\tIt should be between 4 and 511 inclusive!\n", tlv->length);

      return XEINVALIDTLV;
    }
  uint32_t medHdr = htonl(*(uint32_t *)&tlv->info_string[0]);
  uint32_t oui=0;
  uint32_t subtype=0;
  //fetching the Organizational Unique Identifier from MED header
  oui = medHdr & 0xFFFFFF00;
  oui=oui>>8;
  //extracting the subtype
  subtype = medHdr & 0x000000FF;
  if (oui==MED_OUI)
  {
     if(subtype == 1)
     {
	   // LLDP-MED Capabilities TLV
	   ret=validate_medCapabilities_tlv(tlv);
	 } else if(subtype == 2){
	   // Network Policy TLV
	   ret=validate_networkPolicy_tlv(tlv);
     } else if (subtype == 4){
       // Extended Power-via-MDI TLV
       ret=validate_extendedPower_tlv(tlv); 
     } else {
 		    PRINT("\n This MED TLV is not supported\n");
	   ret = XUNRECOGTLV;
     }		  
  } else if (oui==ORG802_OUI) {
    if(subtype == 1)
    {
      // It's MAC/PHY Configuration/Status TLV
     ret= validate_macPhy_configStatus_tlv(tlv);
    } else {
      PRINT("\n This Organizational TLV is not supported\n");
	  ret = XUNRECOGTLV;
    }
  } else {
     PRINT("\n This Organizational TLV is not supported\n");
	 ret = XUNRECOGTLV;
  }

  return ret;
}

uint8 validate_generic_tlv(struct lldp_tlv *tlv)
{
  //debug_printf(DEBUG_TLV, "Generic TLV Validation for TLV type: %d.\n", tlv->type);
  //debug_printf(DEBUG_TLV, "TLV Info String Length: %d\n", tlv->length);
  //debug_printf(DEBUG_TLV, "TLV Info String: ");
  //debug_hex_dump(DEBUG_TLV, tlv->info_string, tlv->length);

  // Length will never fall below 0 because it's an unsigned variable
  if(tlv->length > 511)
    {
      //debug_printf(DEBUG_NORMAL, "[ERROR] TLV has invalid length '%d'.\n\tIt should be between 0 and 511 inclusive!\n", tlv->length);
      
      return XEINVALIDTLV;
    }

  return XVALIDTLV;
}

// A helper function to flatten an exploded TLV.
struct lldp_flat_tlv *flatten_tlv(struct lldp_tlv *tlv) {
    PRINT("\nIN->fLatten_tlv\n");
 
  if(tlv == NULL) {
    PRINT("\n tlv is NULLLLL!!!\n");
    return NULL;
  } else
    PRINT("\ntlv not null\n");
    PRINT("\ntlv type %d\n",tlv->type);


  // Initialize to 0 here... helps memory checkers like valgrind have better accuracy.
  uint16_t type_and_length       = 0;
  struct lldp_flat_tlv *flat_tlv = NULL;

  // Munge our type and length into a single container.
  type_and_length = tlv->type;

  /*
  //debug_printf(DEBUG_TLV, "[FLATTEN] Flatting TLV Type: %d\n", tlv->type);
  //debug_hex_dump(DEBUG_TLV, tlv->info_string, tlv->length);
  //debug_printf(DEBUG_TLV, "Flattening TLV: \n");
  //debug_printf(DEBUG_TLV, "\tTLV Type: %d\n", tlv->type);
  //debug_printf(DEBUG_TLV, "\tTLV Length: %d\n", tlv->length);*/
    
  type_and_length = type_and_length << 9;
  PRINT("\nTLV LENGTH=%d\n",tlv->length);
  type_and_length |= tlv->length;
  
  //debug_printf(DEBUG_TLV, "Before Network Byte Order: ");
  //debug_hex_PRINT(DEBUG_TLV, (uint8 *)&type_and_length, sizeof(type_and_length));

  // Convert it to network byte order...
  type_and_length = htons(type_and_length);

  //debug_printf(DEBUG_TLV, "After Network Byte Order: ");
  //debug_hex_PRINT(DEBUG_TLV, (uint8 *)&type_and_length, sizeof(type_and_length));

  // Now cram all of the bits into our flat TLV container.
  flat_tlv = malloc(sizeof(struct lldp_flat_tlv));
  
  if(flat_tlv) {
    // We malloc for the size of the entire TLV, plus the 2 bytes for type and length.
    flat_tlv->size = tlv->length + 2;

    flat_tlv->tlv = malloc(flat_tlv->size);
    bzero(&flat_tlv->tlv[0], flat_tlv->size);

    //debug_printf(DEBUG_TLV, "Flattened TLV: ");
    //debug_hex_dump(DEBUG_TLV, flat_tlv->tlv, flat_tlv->size);

    // First, copy in the type and length
    memcpy(&flat_tlv->tlv[0], &type_and_length, sizeof(type_and_length));
    
    //debug_printf(DEBUG_TLV, "Flattened TLV: ");
    //debug_hex_dump(DEBUG_TLV, flat_tlv->tlv, flat_tlv->size);

    // Then copy in the info string, for the size of tlv->length
    memcpy(&flat_tlv->tlv[sizeof(type_and_length)], tlv->info_string, tlv->length);

    // We're done. ;)

  } else {
    //debug_printf(DEBUG_NORMAL, "[ERROR] Unable to malloc buffer in %s() at line: %d!\n", __FUNCTION__, __LINE__);
  }

  //debug_printf(DEBUG_TLV, "Flattened TLV: ");
  //debug_hex_dump(DEBUG_TLV, flat_tlv->tlv, flat_tlv->size);

  PRINT("\nOUT->fLatten_tlv\n");
  return flat_tlv;
}

// A helper function to explode a flattened TLV.
struct lldp_tlv *explode_tlv(struct lldp_flat_tlv *flat_tlv) {
  
  uint16_t type_and_length = 0;
  struct lldp_tlv *tlv   = NULL;

  tlv = malloc(sizeof(struct lldp_tlv));

  if(tlv) {

    // Suck the type and length out...
    type_and_length = *(uint16_t *)&tlv[0];

    tlv->length     = type_and_length & 511;
    type_and_length = type_and_length >> 9;
    tlv->type       = type_and_length;

    tlv->info_string = malloc(tlv->length);

    if(tlv->info_string) {
      // Copy the info string into our TLV...
      memcpy(&tlv->info_string[0], &flat_tlv->tlv[sizeof(type_and_length)], tlv->length);
    } else { // tlv->info_string == NULL
      //debug_printf(DEBUG_NORMAL, "[ERROR] Unable to malloc buffer in %s() at line: %d!\n", __FUNCTION__, __LINE__);
    }
  } else { // tlv == NULL
    //debug_printf(DEBUG_NORMAL, "[ERROR] Unable to malloc buffer in %s() at line: %d!\n", __FUNCTION__, __LINE__);    
  }

  return tlv;
}

uint8 tlvcpy(struct lldp_tlv *dst, struct lldp_tlv *src)
{
  //LLDP_FUNCTION_DEBUG;
  
  if(dst != NULL && src != NULL)
    {
      dst->type = src->type;
      dst->length = src->length;
      dst->info_string = malloc(dst->length);
      
      if(dst->info_string != NULL)
	{
	  memcpy(dst->info_string, src->info_string, dst->length);
	}
      else
	{
	  //debug_printf(DEBUG_NORMAL, "[ERROR] Couldn't allocate memory!!\n");

	}
    }
  else
    {
      //debug_printf(DEBUG_NORMAL, "[ERROR] NULL TLV!\n");
    }

  return 0;
}

struct lldp_tlv *tlvpop(uint8 *buffer)
{
  //LLDP_FUNCTION_DEBUG;

  return NULL;
}

uint8 tlvpush(uint8 *buffer, struct lldp_tlv *tlv)
{
  //LLDP_FUNCTION_DEBUG;

  return -1;
}

uint8 lldp_cache_tlv(struct lldp_tlv *tlv)
{
  //LLDP_FUNCTION_DEBUG;

  return -1;
}

void destroy_tlv_list(struct lldp_tlv_list **tlv_list) {
  struct lldp_tlv_list *current  = *tlv_list;

  while(current != NULL) {
    current = current->next;
    destroy_tlv((*tlv_list)->tlv);
    free((*tlv_list));
    (*tlv_list) = current;
  }

  (*tlv_list) = NULL;
}

#if 0
/* Iterates through the global MSAP cache list to find a specific MSAP cache */
/* The MSAP caches are basically per-device TLV records, so a given cache */
/* represents a specific device */
struct lldp_msap_cache *lldp_msap_cache(struct lldp_port *lldp_port, uint8 *msap, uint64 msap_length)
{
  //struct lldp_msap_cache *msap_cache = lldp_port->rx.msap; //COMMENT XXX
  struct lldp_msap_cache *msap_cache = NULL;  //Temporarily put for commenting the first line

  //LLDP_FUNCTION_DEBUG;

  while(msap_cache != NULL)
    {
      if(memcmp(msap_cache->msap, msap, msap_length) == 0)
	{
	  /* Do some extra checking here for the heck of it */
	  if(msap_length != msap_cache->msap_length)
	    {
	      //debug_printf(DEBUG_NORMAL, "[WARNING] MSAP length mismatch between cache length (%d) and query length (%d).\n", msap_cache->msap_length, msap_length);
	    }

	  //debug_printf(DEBUG_TLV, "Found matching MSAP cache: \n");

	  //debug_hex_dump(DEBUG_TLV, msap_cache->msap, msap_cache->msap_length);

	  return msap_cache;
	}
      
      msap_cache = msap_cache->next;
    }

  return NULL;
}


void destroy_msap_cache(struct lldp_msap_cache *msap_cache) {
  //LLDP_FUNCTION_DEBUG;

}

uint8 update_msap_cache(struct lldp_port *lldp_port, struct lldp_msap_cache *msap_cache)
{
  //LLDP_FUNCTION_DEBUG;

  /*  if(msap_cache != NULL)
    {
      //TODO: Run through each TLV in the global cache for this MSAP 
      // And update the value
    }
  else
    {
      //debug_printf(DEBUG_TLV, "MSAP Cache Miss for %d byte MSAP: \n", msap_length);

      //debug_hex_dump(DEBUG_TLV, msap, msap_length);

      //create_msap_cache(lldp_port, msap, msap_length, tlv_list);
    }*/

  return 0;
}

uint8 create_msap_cache(struct lldp_port *lldp_port, uint8 *msap, uint64 msap_length, struct lldp_tlv_list *tlv_list)
{
  struct lldp_msap_cache *msap_cache = NULL;

  //LLDP_FUNCTION_DEBUG;

  msap_cache = malloc(sizeof(struct lldp_msap_cache));

  if(msap_cache == NULL)
    {
      //debug_printf(DEBUG_NORMAL, "[ERROR] Unable to malloc buffer in %s() at line: %d!\n", __FUNCTION__, __LINE__);      
    }

  msap_cache->msap_length = msap_length;
  
  msap_cache->msap = malloc(msap_length);

  memcpy(msap_cache->msap, msap, msap_length);

  msap_cache->tlv_list = tlv_list;

  msap_cache->next = NULL;
#if 0
  if(lldp_port->rx.msap == NULL)
    {
      lldp_port->rx.msap = msap_cache;
    }
  else
    {
      //debug_printf(DEBUG_NORMAL, "[WARNING] MSAP caches larger than 1 device are not yet supported.\n");
    }
#endif //COMMENT XXX
  return 0;
}

#endif

void add_tlv(struct lldp_tlv *tlv, struct lldp_tlv_list **tlv_list) {
  struct lldp_tlv_list *tail = malloc(sizeof(struct lldp_tlv_list));
  struct lldp_tlv_list *tmp = *tlv_list;

  PRINT("\nIN->add_tlv\n");
  tail->tlv  = tlv;
  //memcpy(&(tail->tlv),tlv,sizeof(struct lldp_tlv));
  tail->next = NULL;

  if((*tlv_list) == NULL) {
    (*tlv_list) = tail;
  } else {
    while(tmp->next != NULL) {
      tmp = tmp->next;      
    }    

    tmp->next = tail;
  }
  PRINT("\nOUT->add_tlv\n");
}
